@@ -6,9 +6,10 @@ A tiny web type setter.
 (c) Daniel Krajzewicz 2020
 daniel@krajzewicz.de
 http://www.krajzewicz.de
+https://github.com/dkrajzew/degrotesque
 http://www.krajzewicz.de/blog/degrotesque.php
 
-Available under GPL 3.0, all rights reserved
+Available under LGPL 3.0 or later, all rights reserved
 """
 
 
@@ -23,19 +24,19 @@ actionsDB = {
  # english quotes
  "quotes.english": [
    [[u"(\\s+)'", u"'"], [u"\\1&lsquo;", u"&rsquo;"]],
-   [[u"\"", u"\""], [u"&ldquo;", u"&rdquo;"]],
+   [[u"\"", u"\""], [u"&ldquo;", u"&rdquo;"]]
   ],
 
  # french quotes
  "quotes.french": [
    [[u"&lt;&lt;", u"&gt;&gt;"], [u"&laquo;", u"&raquo;"]],
-   [[u"&lt;", u"&gt;"], [u"&lsaquo;", u"&rsaquo;"]],
+   [[u"&lt;", u"&gt;"], [u"&lsaquo;", u"&rsaquo;"]]
   ],
   
  # german quotes
  "quotes.german": [
    [[u"(\\s+)'", u"'"], [u"\\1&sbquo;", u"&rsquo;"]],
-   [[u"\"", u"\""], [u"&bdquo;", u"&rdquo;"]],
+   [[u"\"", u"\""], [u"&bdquo;", u"&rdquo;"]]
   ],
   
  # conversion to HTML quotes (<q>)
@@ -43,38 +44,38 @@ actionsDB = {
    [[u"(\\s+)'", u"'"], [u"\\1<q>", u"</q>"]],
    [[u"\"", u"\""], [u"<q>", u"</q>"]],
    [[u"&lt;&lt;", u"&gt;&gt;"], [u"<q>", u"</q>"]],
-   [[u"&lt;", u"&gt;"], [u"<q>", u"</q>"]],
+   [[u"&lt;", u"&gt;"], [u"<q>", u"</q>"]]
   ],
   
  # commercial signs
  "commercial": [
    [[u"\\([c|C]\\)", None], [u"&copy;", None]],
    [[u"\\([r|R]\\)", None], [u"&reg;", None]],
-   [[u"\\([t|T][m|M]\\)", None], [u"&trade;", None]],
+   [[u"\\([t|T][m|M]\\)", None], [u"&trade;", None]]
   ],
   
  # dashes
  "dashes": [
    # missing: ndash for number ranges 
    [[u"(\\s+)-(\\s+)", None], [u"\\1&mdash;\\2", None]],
-   [[u"([\\d]+)-([\\d]+)", None], [u"\\1&ndash;\\2", None]],
+   [[u"([\\d]+)-([\\d]+)", None], [u"\\1&ndash;\\2", None]]
    #[[u"\\W-([\\d]+)", None], [u"&ndash;\\1", None]],
    #[[u"([\\d]+)-\\W", None], [u"\\1&ndash;", None]],
   ],
 
  # bullets
  "bullets": [
-   [[u"\\*", None], [u"&bull;", None]],
+   [[u"\\*", None], [u"&bull;", None]]
   ],
     
  # ellipsis
  "ellipsis": [
-   [[u"\\.\\.\\.", None], [u"&hellip;", None]],
+   [[u"\\.\\.\\.", None], [u"&hellip;", None]]
   ],
     
  # apostrophe
  "apostrophe": [
-   [[u"'", None], [u"&apos;", None]],
+   [[u"'", None], [u"&apos;", None]]
   ],
     
  # math signs
@@ -90,13 +91,21 @@ actionsDB = {
    [[u"&gt;=", None], [u"&ge;", None]],
    [[u"([\\d]+)(\\s*)\*(\\s*)([\\d]+)", None], [u"\\1\\2&times;\\3\\4", None]],
    [[u"([\\d]+)(\\s*)x(\\s*)([\\d]+)", None], [u"\\1\\2&times;\\3\\4", None]],
-   [[u"([\\d]+)(\\s*)/(\\s*)([\\d]+)", None], [u"\\1\\2&divide;\\3\\4", None]],
+   [[u"([\\d]+)(\\s*)/(\\s*)([\\d]+)", None], [u"\\1\\2&divide;\\3\\4", None]]
   ],
     
  # dagger
  "dagger": [
    [[u"\\*\\*", None], [u"&Dagger;", None]],
-   [[u"\\*", None], [u"&dagger;", None]],
+   [[u"\\*", None], [u"&dagger;", None]]
+  ],    
+
+ # masks
+ "masks": [
+   [[u"978-([\\d]+)-([\\d]+)-([\\d]+)-([\\d])(\\D)", None], [u"978-\\1-\\2-\\3-\\4\\5", None]],
+   [[u"979-([\\d]+)-([\\d]+)-([\\d]+)-([\\d])(\\D)", None], [u"979-\\1-\\2-\\3-\\4\\5", None]],
+   [[u"([\\d]+)-([\\d]+)-([\\d]+)-([\\d])(\\D)", None], [u"\\1-\\2-\\3-\\4\\5", None]],
+   [[u"ISSN (\\d{4})-(\\d{4})(\\D)", None], [u"ISSN \\1-\\2\\3", None]]
   ]    
 }
 
@@ -134,12 +143,51 @@ class Degrotesque():
        u"!--"
      ]
      # the actions to apply
+     self.restoreDefaultActions()
+     
+     
+  # --- restoreDefaultActions
+  def restoreDefaultActions(self):
+     """Instantiates default actions"""     
      self._actions = []
+     self._actions.extend(actionsDB["masks"])
      self._actions.extend(actionsDB["quotes.english"])
      self._actions.extend(actionsDB["dashes"])
      self._actions.extend(actionsDB["ellipsis"])
      self._actions.extend(actionsDB["math"])
      self._actions.extend(actionsDB["apostrophe"])
+  
+    
+  # --- setActions
+  def setActions(self, actNames):
+    """Returns the actions to apply.
+       If the given names of actions are None or empty, the default actions 
+       are used.
+       Otherwise, the actions matching the given names are retrieved from the
+       internal database and their list is returned.
+       :param actNames The names of the actions to use (or None if default 
+                       actions shall be used)"""
+    if actNames==None or len(actNames)==0:
+      return
+    actNames = actNames.split(",")
+    self._actions = []
+    for an in actNames:
+      if an in actionsDB:
+        self._actions.extend(actionsDB[an])
+      else:
+        raise ValueError("Action '%s' is not known." % (an))
+
+
+  # --- setToSkip
+  def setToSkip(self, toSkipNames):
+    """Returns the elements which contents shall not be changed.
+       If the given names of elements are None or empty, the default elements 
+       to skip are used.
+       Otherwise, a list with the elements to skip is built.
+       :param toSkipNames The names of elements which shall not be changed"""
+    if toSkipNames==None or len(toSkipNames)==0:
+      return 
+    self._elementsToSkip = [x.strip() for x in toSkipNames.split(',')] 
 
 
   # --- _getTagName
@@ -227,7 +275,8 @@ class Degrotesque():
        :param actions The actions to apply"""
     i = 0
     # extract text parts
-    marks = self._mark(html)
+    lowerHTML = html.lower()
+    marks = self._mark(lowerHTML)
     assert(len(html)==len(marks))
     while i<len(html):
       if marks[i]=="1":
@@ -261,41 +310,10 @@ class Degrotesque():
         html = html[:i] + tmp
         marks = marks[:i] + "0"*l + marks[i+opening.end():]
         assert(len(html)==len(marks))
+        i = i + opening.end() - 1
         break
       i = i + 1
     return html
-  
-    
-  # --- setActions
-  def setActions(self, actNames):
-    """Returns the actions to apply.
-       If the given names of actions are None or empty, the default actions 
-       are used.
-       Otherwise, the actions matching the given names are retrieved from the
-       internal database and their list is returned.
-       :param actNames The names of the actions to use (or None if default 
-                       actions shall be used)"""
-    if actNames==None or len(actNames)==0:
-      return
-    actNames = actNames.split(",")
-    self._actions = []
-    for an in actNames:
-      if an in actionsDB:
-        self._actions.extend(actionsDB[an])
-      else:
-        raise ValueError("Action '%s' is not known." % (an))
-
-
-  # --- setToSkip
-  def setToSkip(self, toSkipNames):
-    """Returns the elements which contents shall not be changed.
-       If the given names of elements are None or empty, the default elements 
-       to skip are used.
-       Otherwise, a list with the elements to skip is built.
-       :param toSkipNames The names of elements which shall not be changed"""
-    if toSkipNames==None or len(toSkipNames)==0:
-      return 
-    self._elementsToSkip = [x.strip() for x in toSkipNames.split(',')] 
 
 
 
